#!/usr/bin/python

import gtk
import xdg.Menu
import xdg.DesktopEntry

COL_PATH = 0
COL_PIXBUF = 1
COL_IS_DIRECTORY = 2


class PyApp(gtk.Window): 
	def __init__(self):
		super(PyApp, self).__init__()
		
		self.current_menu = "Top"
		
		self.set_size_request(365, 500)
		self.maximize()
		
		self.connect("destroy", gtk.main_quit)
		self.set_title("Main Menu")
		
		vbox = gtk.VBox(False, 0);
		
		toolbar = gtk.Toolbar()
		vbox.pack_start(toolbar, False, False, 0)
		
		self.upButton = gtk.ToolButton(gtk.STOCK_GO_UP);
		self.upButton.set_is_important(True)
		self.upButton.set_sensitive(True)
		toolbar.insert(self.upButton, -1)
		
		toolspacer = gtk.SeparatorToolItem()
		toolspacer.set_draw(False)
		toolspacer.set_expand(True)
		toolbar.insert(toolspacer, -1)
		
		closebutton = gtk.ToolButton(gtk.STOCK_CANCEL)
		closebutton.set_is_important(True)
		closebutton.set_sensitive(True)
		toolbar.insert(closebutton, -1)
		
		self.dirIcon = self.get_icon(gtk.STOCK_DIRECTORY)
		self.appIcon = self.get_icon(gtk.STOCK_EXECUTE)
		
		sw = gtk.ScrolledWindow()
		sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		vbox.pack_start(sw, True, True, 0)
		
		self.store = self.create_store()
		self.fill_store()
		
		self.iconView = gtk.IconView(self.store)
		self.iconView.set_selection_mode(gtk.SELECTION_SINGLE)
		
		self.upButton.connect("clicked", self.on_up_clicked)
		closebutton.connect("clicked", self.on_closebutton_clicked)
		self.connect('key_press_event', self.keypress)
		
		self.iconView.set_text_column(COL_PATH)
		self.iconView.set_pixbuf_column(COL_PIXBUF)
		
		self.iconView.connect("selection-changed", self.on_item_activated)
		sw.add(self.iconView)
		self.iconView.grab_focus()
		
		self.add(vbox)
		self.show_all()
	
	def get_icon(self, name):
		theme = gtk.icon_theme_get_default()
		return theme.load_icon(name, 48, 0)
	
	def create_store(self):
		store = gtk.ListStore(str, gtk.gdk.Pixbuf, bool)
		store.set_sort_column_id(COL_PATH, gtk.SORT_ASCENDING)
		return store
	
	def fill_store(self):
		self.store.clear()
		
		if self.current_menu == "Top":
			print "Displaying top menu."
			for entry in menu.getEntries():
				if isinstance(entry, xdg.Menu.Menu):
					self.store.append([entry, self.dirIcon, True])
		else:
			print "Not yet implemented, but if it was I'd be showing the " + self.current_menu + " menu."
			for entry in menu.getEntries():
				if isinstance(entry, xdg.Menu.MenuEntry):
					print entry
	
	def on_item_activated(self, widget):
		selected = self.iconView.get_selected_items()
		if len(selected) != 0:
			i = selected[0][0]
			category = self.store[i][0]
			self.current_menu = category
			self.fill_store()
	
	def on_up_clicked(self, widget):
	        print "Whee! :D  We're going up! :)"
		self.current_menu = "Top"
		self.fill_store()
	
	def on_closebutton_clicked(self, widget):
		print "I guess we all die at some point, right?"
		gtk.main_quit()

	def keypress(self, widget, event):
		if event.keyval == gtk.keysyms.Escape:
			print "Escape! Escape!"
			self.on_closebutton_clicked("")
    

menu = xdg.Menu.parse()
PyApp()
gtk.main()
