#!/usr/bin/env python

import os
import gtk
import pickle
import webkit
import subprocess
import feedparser

class snf_newsreader_desktop:
    
    def __init__(self):

        # So we can remember whether we're hiding or showing the window
        # from status icon
        self.winNotIconHidden = 0 # 0 for showing

        #Open the file containing the feeds to parse
        self.dataunit = self.loadConfigFile()
        
        #A list containing the feeds that have been fetched so that we do not have
        #to keep bothering the RSS servers (also making the application faster).
        self.feedcontainer = [None]
    
        #Set the Glade file
        builderFile = "main-view.glade"
        self.builder = gtk.Builder()
        self.builder.add_from_file(builderFile)
        self.builder.connect_signals(self)

        #Build our feed view
        self.feed_tree = self.builder.get_object("treeview1")
        self.buildFeedList()
        self.refreshFeeds(self, None)
        self.webbox = self.builder.get_object("webbox")
        self.feedview = webkit.WebView()
        feedscroll = gtk.ScrolledWindow()
        feedscroll.add(self.feedview)
        self.webbox.pack_start(feedscroll, expand=True, fill=True, padding=0)
        feedscroll.show()
        self.feedview.show()
        
        self.mainWindow = self.builder.get_object("mainWindow")
        self.mainWindow.show()
        self.mainWindow.connect("destroy", self.shutdownProcess)

    def menuQuit(self, widget, data=None):
        self.mainWindow.destroy()
        self.shutdownProcess(self, None)
        
    def shutdownProcess(self, widget, data=None):
        self.dumpConfigFile(dataunit=self.dataunit)
        gtk.main_quit()

    def goProjectWebsite(self, widget, data=None):
        self.feedview.open("http://mccullaugh.com/snf-newsreader")

    def noticonWinHideShow(self, widget, data=None):
        if self.winNotIconHidden == 0:
            self.mainWindow.hide()
            self.winNotIconHidden = 1
        else:
            self.mainWindow.show()
            self.winNotIconHidden = 0

    def loadConfigFile(self):
        filename = os.environ['HOME']
        filename += "/.snf-newsreader-rc"
        print "Loading configuration: ", filename
        if not os.path.exists(filename):
            subprocess.call(["touch", filename])
            self.dumpConfigFile(dataunit=[["http://rss.slashdot.org/Slashdot/slashdot", "Slashdot", 3], ["http://feeds.theonion.com/theonion/daily","The Onion", 3]])
        pickleFileName = filename
        pickleFile = open(pickleFileName, 'rb')
        returndata = pickle.load(pickleFile)
        pickleFile.close()
        return returndata

    def dumpConfigFile(self, dataunit):
        filename = os.environ['HOME']
        filename += "/.snf-newsreader-rc"
        pickleFileName = filename
        pickleFile = open(pickleFileName, 'wb')
        pickle.dump(dataunit, pickleFile, pickle.HIGHEST_PROTOCOL)
        pickleFile.close()

    def addNewFeedDialog(self, widget, data=None):
        self.newFeedDialog = self.builder.get_object("newFeedDialog")
        feedURL = self.builder.get_object("feedURL")
        customFeedName = self.builder.get_object("customFeedName")
        fetchItemsAmount = self.builder.get_object("fetchItemsAmount")
        feedURL.set_text("")
        customFeedName.set_text("")
        fetchItemsAmount.set_value(3)
        addEditButton = self.builder.get_object("addEditButton")
        addEditButton.connect("clicked", self.addNewFeed)
        self.newFeedDialog.show()

    def cancelNewFeedDialog(self, widget, data=None):
        self.newFeedDialog.hide()

    def addNewFeed(self, widget, data=None):
        print "I'm the add function."
        feedURL = self.builder.get_object("feedURL").get_text()
        customFeedName = self.builder.get_object("customFeedName").get_text()
        itemsAmount = self.builder.get_object("fetchItemsAmount").get_value_as_int()
        print feedURL
        print customFeedName
        if feedURL == "":
            print "Failure to add: blank URL field"
        else:
            self.dataunit.append([feedURL, customFeedName, itemsAmount])
            self.newFeedDialog.hide()
            newFeedRow = self.feed_store.append(None, [customFeedName])
            self.feed_store.append(newFeedRow, ["News not yet fetched"])
            self.feedcontainer.append(feedparser.parse(feedURL))
            self.updateFeedList(None)


    def editFeedPreferencesDialog(self, widget, data=None):
        selection = self.feed_tree.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        tree_model, tree_iter = selection.get_selected()
        selectedFeed = self.feed_store.get_path(tree_iter)[0]
        self.newFeedDialog = self.builder.get_object("newFeedDialog")
        feedURL = self.builder.get_object("feedURL")
        customFeedName = self.builder.get_object("customFeedName")
        fetchItemsAmount = self.builder.get_object("fetchItemsAmount")
        feedURL.set_text(self.dataunit[selectedFeed][0])
        customFeedName.set_text(self.dataunit[selectedFeed][1])
        fetchItemsAmount.set_value(self.dataunit[selectedFeed][2])
        addEditButton = self.builder.get_object("addEditButton")
        addEditButton.connect("clicked", self.editFeedPreferences, selectedFeed)
        self.newFeedDialog.show()

    def editFeedPreferences(self, widget, selectedFeed):
        feedURL = self.builder.get_object("feedURL").get_text()
        customFeedName = self.builder.get_object("customFeedName").get_text()
        itemsAmount = self.builder.get_object("fetchItemsAmount").get_value_as_int()
        print feedURL
        print customFeedName
        if feedURL == "":
            print "Failure to add: blank URL field"
        else:
            self.dataunit[selectedFeed][0] = feedURL
            self.dataunit[selectedFeed][1] = customFeedName
            self.dataunit[selectedFeed][2] = itemsAmount
            self.newFeedDialog.hide()
            self.updateFeedList(None)

    def buildFeedList(self):
        self.feed_store = gtk.TreeStore(str)
        for feed in self.dataunit:
            feedrow = self.feed_store.append(None, [feed[1]])
            self.feed_store.append(feedrow, ["News not yet fetched"])
        self.feed_name_renderer = gtk.CellRendererText()
        self.feed_list_view = gtk.TreeViewColumn('News Feed List', self.feed_name_renderer, text=0)
        self.feed_tree.set_model(self.feed_store)
        self.feed_tree.append_column(self.feed_list_view)
        
    def refreshFeeds(self, widget, data=None):
        print self.dataunit
        self.feedcontainer = []
        for feed in self.dataunit:
	    self.feedcontainer.append(feedparser.parse(feed[0]))
        self.updateFeedList(self, None)

    def updateFeedList(self, widget, data=None):
        feeds = 0
        for feed in self.dataunit:
            parsefeed = self.feedcontainer[feeds]
            feediter = self.feed_store.get_iter(feeds)
            while not self.feed_store.iter_children(feediter) is None:
                self.feed_store.remove(self.feed_store.iter_children(feediter))
            topLimit = feed[2]
            for feedItem in parsefeed["items"]:
                if topLimit == 0:
                    break
                self.feed_store.append(feediter, [feedItem["title"]])
                topLimit -= 1
            feeds += 1

    def deleteFeed(self, widget, data=None):
        selection = self.feed_tree.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        tree_model, tree_iter = selection.get_selected()
        selectedFeed = self.feed_store.get_path(tree_iter)[0]
        del self.dataunit[selectedFeed]
        del self.feedcontainer[selectedFeed]
        deadFeedIter = self.feed_store.get_iter(selectedFeed)
        self.feed_store.remove(deadFeedIter)
        self.updateFeedList(None)

    def showAboutBox(self, widget, data=None):
        aboutdialog1 = self.builder.get_object("aboutdialog1")
        aboutdialog1.show()

    def newsItemSelected(self, widget, data=None):
        selection = self.feed_tree.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        tree_model, tree_iter = selection.get_selected()
        parsefeed = self.feedcontainer[self.feed_store.get_path(tree_iter)[0]]
        self.feedview.load_html_string(parsefeed["items"][self.feed_store.get_path(tree_iter)[1]]["summary"], "file:///")

    def goArticleSite(self, widget, data=None):
        selection = self.feed_tree.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        tree_model, tree_iter = selection.get_selected()
        parsefeed = self.feedcontainer[self.feed_store.get_path(tree_iter)[0]]
        self.feedview.open(parsefeed["items"][self.feed_store.get_path(tree_iter)[1]]["link"])
        
if __name__ == "__main__":
    news = snf_newsreader_desktop()
    gtk.main()
